@@ -1,28 +1,43 @@
 /*
-    SPDX-FileCopyrightText: 2008, 2011 Will Stephenson <wstephenson@kde.org>
-    SPDX-FileCopyrightText: 2011-2013 Lamarque V. Souza <lamarque@kde.org>
-    SPDX-FileCopyrightText: 2013 Daniel Nicoletti <dantti12@gmail.com>
-    SPDX-FileCopyrightText: 2013 Jan Grulich <jgrulich@redhat.com>
+    Copyright 2008,2011 Will Stephenson <wstephenson@kde.org>
+    Copyright 2011-2013 Lamarque V. Souza <lamarque@kde.org>
+    Copyright 2013 Daniel Nicoletti <dantti12@gmail.com>
+    Copyright 2013 Jan Grulich <jgrulich@redhat.com>
 
-    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) version 3, or any
+    later version accepted by the membership of KDE e.V. (or its
+    successor approved by the membership of KDE e.V.), which shall
+    act as a proxy defined in Section 6 of version 3 of the license.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef NETWORKMANAGERQT_DEVICE_H
 #define NETWORKMANAGERQT_DEVICE_H
 
 #include <QObject>
+#include <QtCore/QtGlobal>
 #include <QSharedPointer>
 
-#include "activeconnection.h"
-#include "devicestatistics.h"
-#include "dhcp4config.h"
-#include "dhcp6config.h"
+#include <networkmanagerqt/networkmanagerqt_export.h>
 #include "generictypes.h"
 #include "ipconfig.h"
-#include <networkmanagerqt/networkmanagerqt_export.h>
+#include "dhcp4config.h"
+#include "dhcp6config.h"
+#include "activeconnection.h"
 
 namespace NetworkManager
 {
+
 class DevicePrivate;
 class DeviceStateReason;
 class DeviceStateReasonPrivate;
@@ -50,8 +65,9 @@ class NETWORKMANAGERQT_EXPORT Device : public QObject
     Q_PROPERTY(bool firmwareMissing READ firmwareMissing)
     Q_PROPERTY(bool autoconnect READ autoconnect WRITE setAutoconnect)
     Q_PROPERTY(DeviceStateReason stateReason READ stateReason)
+
+    //Q_PROPERTY(Solid::Control::IPv4Config ipV4Config READ ipV4Config WRITE setIpV4Config)
     Q_PROPERTY(State state READ state)
-    Q_PROPERTY(NetworkManager::DeviceStatistics::Ptr deviceStatistics READ deviceStatistics)
 
 public:
     typedef QSharedPointer<Device> Ptr;
@@ -75,7 +91,7 @@ public:
         WaitingForSecondaries = 90, /**< The device is waiting for secondary connections to be activated */
         Activated = 100, /**< The device is active */
         Deactivating = 110, /**< The device's network connection is being torn down */
-        Failed = 120, /**< The device is in a failure state following an attempt to activate it */
+        Failed = 120 /**< The device is in a failure state following an attempt to activate it */
     };
     Q_ENUM(State)
 
@@ -83,89 +99,42 @@ public:
      * Enums describing the reason for a connection state change
      * @note StateChangeReasons NewActivation, ParentChanged, ParentManagedChanged are available in runtime NM >= 1.0.4
      */
-    enum StateChangeReason {
-        UnknownReason = 0,
-        NoReason = 1,
-        NowManagedReason = 2,
-        NowUnmanagedReason = 3,
-        ConfigFailedReason = 4,
-        ConfigUnavailableReason = 5,
-        ConfigExpiredReason = 6,
-        NoSecretsReason = 7,
-        AuthSupplicantDisconnectReason = 8,
-        AuthSupplicantConfigFailedReason = 9,
-        AuthSupplicantFailedReason = 10,
-        AuthSupplicantTimeoutReason = 11,
-        PppStartFailedReason = 12,
-        PppDisconnectReason = 13,
-        PppFailedReason = 14,
-        DhcpStartFailedReason = 15,
-        DhcpErrorReason = 16,
-        DhcpFailedReason = 17,
-        SharedStartFailedReason = 18,
-        SharedFailedReason = 19,
-        AutoIpStartFailedReason = 20,
-        AutoIpErrorReason = 21,
-        AutoIpFailedReason = 22,
-        ModemBusyReason = 23,
-        ModemNoDialToneReason = 24,
-        ModemNoCarrierReason = 25,
-        ModemDialTimeoutReason = 26,
-        ModemDialFailedReason = 27,
-        ModemInitFailedReason = 28,
-        GsmApnSelectFailedReason = 29,
-        GsmNotSearchingReason = 30,
-        GsmRegistrationDeniedReason = 31,
-        GsmRegistrationTimeoutReason = 32,
-        GsmRegistrationFailedReason = 33,
-        GsmPinCheckFailedReason = 34,
-        FirmwareMissingReason = 35,
-        DeviceRemovedReason = 36,
-        SleepingReason = 37,
-        ConnectionRemovedReason = 38,
-        UserRequestedReason = 39,
-        CarrierReason = 40,
-        ConnectionAssumedReason = 41,
-        SupplicantAvailableReason = 42,
-        ModemNotFoundReason = 43,
-        BluetoothFailedReason = 44,
-        GsmSimNotInserted = 45,
-        GsmSimPinRequired = 46,
-        GsmSimPukRequired = 47,
-        GsmSimWrong = 48,
-        InfiniBandMode = 49,
-        DependencyFailed = 50,
-        Br2684Failed = 51,
-        ModemManagerUnavailable = 52,
-        SsidNotFound = 53,
-        SecondaryConnectionFailed = 54,
-        DcbFcoeFailed = 55,
-        TeamdControlFailed = 56,
-        ModemFailed = 57,
-        ModemAvailable = 58,
-        SimPinIncorrect = 59,
-        NewActivation = 60,
-        ParentChanged = 61,
-        ParentManagedChanged = 62,
-        Reserved = 65536,
-    };
-    Q_ENUM(StateChangeReason)
+    enum StateChangeReason { UnknownReason = 0, NoReason = 1 , NowManagedReason = 2, NowUnmanagedReason = 3,
+                             ConfigFailedReason = 4, ConfigUnavailableReason = 5,
+                             ConfigExpiredReason = 6, NoSecretsReason = 7, AuthSupplicantDisconnectReason = 8,
+                             AuthSupplicantConfigFailedReason = 9, AuthSupplicantFailedReason = 10,
+                             AuthSupplicantTimeoutReason = 11, PppStartFailedReason = 12, PppDisconnectReason = 13,
+                             PppFailedReason = 14, DhcpStartFailedReason = 15, DhcpErrorReason = 16, DhcpFailedReason = 17,
+                             SharedStartFailedReason = 18, SharedFailedReason = 19,
+                             AutoIpStartFailedReason = 20, AutoIpErrorReason = 21, AutoIpFailedReason = 22,
+                             ModemBusyReason = 23, ModemNoDialToneReason = 24, ModemNoCarrierReason = 25, ModemDialTimeoutReason = 26,
+                             ModemDialFailedReason = 27, ModemInitFailedReason = 28,
+                             GsmApnSelectFailedReason = 29, GsmNotSearchingReason = 30, GsmRegistrationDeniedReason = 31,
+                             GsmRegistrationTimeoutReason = 32, GsmRegistrationFailedReason = 33,
+                             GsmPinCheckFailedReason = 34, FirmwareMissingReason = 35, DeviceRemovedReason = 36,
+                             SleepingReason = 37, ConnectionRemovedReason = 38, UserRequestedReason = 39, CarrierReason = 40,
+                             ConnectionAssumedReason = 41, SupplicantAvailableReason = 42, ModemNotFoundReason = 43, BluetoothFailedReason = 44,
+                             GsmSimNotInserted = 45, GsmSimPinRequired = 46, GsmSimPukRequired = 47, GsmSimWrong = 48 , InfiniBandMode = 49,
+                             DependencyFailed = 50, Br2684Failed = 51, ModemManagerUnavailable = 52, SsidNotFound = 53, SecondaryConnectionFailed = 54,
+                             DcbFcoeFailed = 55, TeamdControlFailed = 56, ModemFailed = 57, ModemAvailable = 58, SimPinIncorrect = 59,
+                             NewActivation = 60, ParentChanged = 61, ParentManagedChanged = 62,
+                             Reserved = 65536
+                           };
 
     enum MeteredStatus {
-        UnknownStatus = 0, /**< The device metered status is unknown. */
-        Yes = 1, /**< The device is metered and the value was statically set. */
-        No = 2, /**< The device is not metered and the value was statically set. */
-        GuessYes = 3, /**< The device is metered and the value was guessed. */
-        GuessNo = 4, /**< The device is not metered and the value was guessed. */
+        UnknownStatus = 0,  /**< The device metered status is unknown. */
+        Yes = 1,  /**< The device is metered and the value was statically set. */
+        No = 2,  /**< The device is not metered and the value was statically set. */
+        GuessYes = 3,  /**< The device is metered and the value was guessed. */
+        GuessNo = 4  /**< The device is not metered and the value was guessed. */
     };
-    Q_ENUM(MeteredStatus)
 
     /**
      * Possible device capabilities
      */
     enum Capability {
         IsManageable = 0x1, /**< denotes that the device can be controlled by this API */
-        SupportsCarrierDetect = 0x2, /**< the device informs us when it is plugged in to the medium */
+        SupportsCarrierDetect = 0x2 /**< the device informs us when it is plugged in to the medium */
     };
     Q_ENUM(Capability)
     Q_DECLARE_FLAGS(Capabilities, Capability)
@@ -189,24 +158,12 @@ public:
         Vlan = NM_DEVICE_TYPE_VLAN, /**< Vlan virtual device */
         Adsl = NM_DEVICE_TYPE_ADSL, /**< ADSL modem device */
         Bridge = NM_DEVICE_TYPE_BRIDGE, /**< Bridge virtual device */
-        Generic = NM_DEVICE_TYPE_GENERIC, /**< Generic device @since 1.0.0 */
-        Team = NM_DEVICE_TYPE_TEAM, /**< Team master device @since 1.0.0 */
-        Gre, /**< Gre virtual device @since 1.2.0, @deprecated use IpTunnel instead*/
-        MacVlan, /**< MacVlan virtual device @since 1.2.0 */
-        Tun, /**< Tun virtual device @since 1.2.0 */
-        Veth, /**< Veth virtual device @since 1.2.0 */
-        IpTunnel, /**< IP Tunneling Device @since 1.2.0 */
-        VxLan, /**< Vxlan Device @since 1.2.0 */
-        MacSec, /**< MacSec Device @since 1.6.0 */
-        Dummy, /**< Dummy Device @since 1.8.0 */
-        Ppp, /**< Ppp Device @since 1.10 */
-        OvsInterface, /**< OvsInterface Device @since 1.10 */
-        OvsPort, /**< OvsPort Device @since 1.10 */
-        OvsBridge, /**< OvsBridge Device @since 1.10 */
-        Wpan, /**< Wpan Device @since 1.14 */
-        Lowpan, /**< Lowpan Device @since 1.14 */
-        WireGuard, /**< WireGuard Device @since 1.14 */
-        WifiP2P, /**< WifiP2P Device @since 1.16 */
+        Generic = NM_DEVICE_TYPE_GENERIC, /**< Generic device @since 0.9.9.0 */
+        Team = NM_DEVICE_TYPE_TEAM, /**< Team master device @since 0.9.9.0 */
+        Gre, /**< Gre virtual device @since 0.9.9.0 */
+        MacVlan, /**< MacVlan virtual device @since 0.9.9.0 */
+        Tun, /**< Tun virtual device @since 0.9.9.0 */
+        Veth, /**< Veth virtual device @since 0.9.9.0 */
     };
     Q_ENUM(Type)
     Q_DECLARE_FLAGS(Types, Type)
@@ -218,10 +175,11 @@ public:
      * @param path UNI of the device
      */
     explicit Device(const QString &path, QObject *parent = nullptr);
+    Device(DevicePrivate &dd, QObject *parent);
     /**
      * Destroys a device object.
      */
-    ~Device() override;
+    virtual ~Device();
     /**
      * Retrieves the interface type.  This is a virtual function that will return the
      * proper type of all sub-classes.
@@ -271,10 +229,6 @@ public:
      * The firmware version.
      */
     QString firmwareVersion() const;
-    /**
-     * Reapplies connection settings on the interface.
-     */
-    QDBusPendingReply<> reapplyConnection(const NMVariantMapMap &connection, qulonglong version_id, uint flags);
     /**
      * Disconnects a device and prevents the device from automatically
      * activating further connections without user intervention.
@@ -286,6 +240,7 @@ public:
      *
      * @since 5.8.0
      *
+     * @note this is a noop in runtime NM < 1.0.0
      */
     QDBusPendingReply<> deleteInterface();
     /**
@@ -393,6 +348,7 @@ public:
      * The device MTU (maximum transmission unit)
      * @since 0.9.9.0
      *
+     * @note always returns 0 in runtime NM < 1.0.6
      */
     uint mtu() const;
 
@@ -400,6 +356,7 @@ public:
      * @return If TRUE, indicates the NetworkManager plugin for the device is likely
      * missing or misconfigured.
      * @since 5.14.0
+     * @note always returns false in runtime NM < 1.2.0
      */
     bool nmPluginMissing() const;
 
@@ -407,6 +364,7 @@ public:
      * @return Whether the amount of traffic flowing through the device is
      * subject to limitations, for example set by service providers.
      * @since 5.14.0
+     * @note always returns UnknownStatus in runtime NM < 1.0.6
      */
     MeteredStatus metered() const;
 
@@ -419,30 +377,23 @@ public:
     void setAutoconnect(bool autoconnect);
 
     /**
-     * Returns Device Statistics interface
-     */
-    DeviceStatistics::Ptr deviceStatistics() const;
-
-    /**
-     * Retrieves a specialized interface to interact with the device corresponding
-     * to a given device interface.
-     *
-     * @returns a pointer to the device interface if it exists, @p 0 otherwise
-     */
-    template<class DevIface>
-    DevIface *as()
+      * Retrieves a specialized interface to interact with the device corresponding
+      * to a given device interface.
+      *
+      * @returns a pointer to the device interface if it exists, @p 0 otherwise
+      */
+    template <class DevIface> DevIface *as()
     {
         return qobject_cast<DevIface *>(this);
     }
 
     /**
-     * Retrieves a specialized interface to interact with the device corresponding
-     * to a given device interface.
-     *
-     * @returns a pointer to the device interface if it exists, 0 otherwise
-     */
-    template<class DevIface>
-    const DevIface *as() const
+      * Retrieves a specialized interface to interact with the device corresponding
+      * to a given device interface.
+      *
+      * @returns a pointer to the device interface if it exists, 0 otherwise
+      */
+    template <class DevIface> const DevIface *as() const
     {
         return qobject_cast<const DevIface *>(this);
     }
@@ -567,6 +518,7 @@ Q_SIGNALS:
      * Emitted when NmPluginMissing property has changed
      * @since 5.14.0
      * @see nmPluginMissing
+     * @note never emmited in runtime NM < 1.2.0
      */
     void nmPluginMissingChanged(bool nmPluginMissing);
 
@@ -574,6 +526,7 @@ Q_SIGNALS:
      * Emitted when metered property has changed
      * @since 5.14.0
      * @see metered
+     * @note never emmited in runtime NM < 1.0.6
      */
     void meteredChanged(MeteredStatus metered);
 
@@ -593,8 +546,6 @@ Q_SIGNALS:
     void udiChanged();
 
 protected:
-    NETWORKMANAGERQT_NO_EXPORT Device(DevicePrivate &dd, QObject *parent);
-
     DevicePrivate *const d_ptr;
 
 private:
@@ -623,3 +574,4 @@ private:
 }
 
 #endif
+
